import pytest

from django.contrib.auth.models import Permission
from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN

pytestmark = pytest.mark.django_db


def test_admin_user_can_update_ethic(admin_client, ethic):
    response = update_ethic(admin_client, ethic)

    assert response.status_code == HTTP_200_OK


def test_user_with_permission_can_update_ethic(client, user, ethic):
    change_project_permission = Permission.objects.get(codename='change_project')
    user.user_permissions.add(change_project_permission)

    response = update_ethic(client, ethic)

    assert response.status_code == HTTP_200_OK


def test_user_without_permission_cannot_update_ethic(client, ethic):
    response = update_ethic(client, ethic)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_update_ethic(anonymous_client, ethic):
    response = update_ethic(anonymous_client, ethic)

    assert response.status_code == HTTP_403_FORBIDDEN


def update_ethic(client, ethic):
    return client.put(
        reverse('ethic-detail', kwargs=dict(project_pk=ethic.project.pk, pk=ethic.pk)),
        data=dict(code='foobar')
    )



import pytest

from django.contrib.auth.models import Permission
from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN, HTTP_405_METHOD_NOT_ALLOWED

pytestmark = pytest.mark.django_db


def test_retrieve_ethics_is_not_provided(client, ethic):
    response = client.get(
        reverse('ethic-detail', kwargs=dict(project_pk=ethic.project.pk, pk=ethic.pk))
    )

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED


def test_admin_user_can_list_ethics(admin_client, project):
    response = list_ethics(admin_client, project)

    assert response.status_code == HTTP_200_OK


def test_user_with_permission_can_list_ethics(client, user, project):
    view_project_permission = Permission.objects.get(codename='view_project')
    user.user_permissions.add(view_project_permission)

    response = list_ethics(client, project)

    assert response.status_code == HTTP_200_OK


def test_user_without_permission_cannot_list_ethics(client, project):
    response = list_ethics(client, project)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_list_ethics(anonymous_client, project):
    response = list_ethics(anonymous_client, project)

    assert response.status_code == HTTP_403_FORBIDDEN


def list_ethics(client, project):
    return client.get(reverse('ethic-list', kwargs=dict(project_pk=project.pk)))



from rest_framework.exceptions import MethodNotAllowed
from rest_framework.permissions import BasePermission


class EthicPermissions(BasePermission):
    perms_map = {
        'DELETE': ['projects.change_project'],
        'GET': ['projects.view_project'],
        'HEAD': [],
        'OPTIONS': [],
        'POST': ['projects.change_project'],
        'PUT': ['projects.change_project'],
    }

    def has_permission(self, request, view):
        if request.method not in self.perms_map:
            raise MethodNotAllowed(request.method)

        if request.method == 'GET' and view.detail is True:
            raise MethodNotAllowed(request.method)

        perms = self.perms_map[request.method]

        return request.user.has_perms(perms) or request.user.has_perms(perms, view.project)


class EthicFilePermissions(BasePermission):
    perms_map = {
        'DELETE': ['projects.change_project', 'project_ethics.delete_ethicfile'],
        'HEAD': [],
        'OPTIONS': [],
        'POST': ['projects.change_project'],
    }

    def has_permission(self, request, view):
        if request.method not in self.perms_map:
            raise MethodNotAllowed(request.method)

        perms = self.perms_map[request.method]

        return request.user.has_perms(perms) or request.user.has_perms(perms, view.ethic.project)
